/*
 * Lua Commander - Main
 *
 * Copyright (C) 1999-2000, Antonio E. Scuri, Tecgraf/PUC-Rio.
 */


#include <stdlib.h>

#include <iup.h>

#include <lua.h>
#include <luadebug.h>

#include "ldebug.h"
#include "lconsole.h"
#include "lnotepad.h"
#include "utl.h"
#include "utl_iup.h"
#include "utl_load_lib.h"


static void lua_getcwd(void)
{
  char* dir = utlGetCurrentDirectory();
  if (!dir)
    lua_error("could not get current directory.");
  lua_pushstring(dir);
  free(dir);
}

static void lua_chdir(void)
{
  lua_Object str_obj;
  char *str;

  str_obj = lua_getparam(1);
  if (str_obj == LUA_NOOBJECT)
    lua_error("invalid parameter.");

  str = lua_getstring(str_obj);
  if (!str)
    lua_error("invalid string parameter.");

  if (!utlSetCurrentDirectory(str))
    lua_error("could not change directory.");
}

static void lua_loadlib(void)
{
  lua_Object lib, func;
  char *lib_name, *func_name;
  int err;

  lib = lua_getparam(1);
  func = lua_getparam(2);

  lib_name = lua_getstring(lib);
  func_name = lua_getstring(func);

  err = utlLoadFunc(lib_name, func_name);

  if (err == -1)
    lua_error("loadlib: can not load library!");
  if (err == -2)
    lua_error("loadlib: can not load function!");

  luaConsoleEnterMessage("loadlib: library successfully loaded.");
}

/***************************************/
/* Where everything begins and ends... */
/***************************************/

int main(int argc, char* argv[])
{
  /* Initialization of the interface library. */
  IupOpen();

  lua_open();
  
  lua_setdebug(1);

  lua_register("loadlib", lua_loadlib);
  lua_register("getcwd", lua_getcwd);
  lua_register("chdir", lua_chdir);
  lua_dostring("function pwd() print(getcwd()) end");

  utlCreateButtonImages();

  luaConsoleCreate(NULL, IUP_CLOSE);
  luaDebugCreate("luaConsole", IUP_CLOSE);
  luaNotepadCreate("luaConsole", IUP_CLOSE);

  /* Starts the interface message loop,        */
  /* and hold until the main window is closed. */
  IupMainLoop();

  luaNotepadKill();
  luaDebugKill();
  luaConsoleKill();

  utlKillButtonImages();

  /* Terminates the interface library. */
  lua_close();
  IupClose();

  return 1;
}
