/*
 * Lua Commander - File Utilities Functions
 *
 * Copyright (C) 1999-2000, Antonio E. Scuri, Tecgraf/PUC-Rio.
 */


#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <time.h>

#include "utl.h"

#ifdef WIN32
#include <windows.h>
#else
#include <unistd.h>
#include <ctype.h>
#endif

char* utlGetCurrentDirectory(void)
{
  static char buffer[10240];

#ifdef WIN32
  if (GetCurrentDirectory(10240, buffer) != 0)
#else
  if (getcwd(buffer, 10240) != NULL)
#endif
    return utlStrDup(buffer);
  else
    return NULL;
}

int utlSetCurrentDirectory(char* dir)
{
#ifdef WIN32
  return SetCurrentDirectory(dir);
#else
  return chdir(dir);
#endif
}

char* utlFullPath(char* filename)
{
  static char buffer[10240];

#ifdef WIN32
  char *title;
  if (GetFullPathName(filename, 10240, buffer, &title)!=0)
    filename = &buffer[0];
  else
    filename = "";
#else
  if (filename[0] != 0 && filename[0] != '/')
  {
    if (getcwd(buffer, 10240) != NULL)
    {
      if (buffer[strlen(buffer)-1] == '/')
        strcat(buffer, filename);
      else
      {
        strcat(buffer, "/");
        strcat(buffer, filename);
      }

      filename = &buffer[0];
    }
    else
      filename = "";
  }
#endif

  return utlStrDup(filename);
}

char* utlMakeFullPath(char* path, char* filename)
{
  int size_path = strlen(path);
  int size_filename = strlen(filename);
  char *full_path = malloc(size_path + size_filename + 2);
  memcpy(full_path, path, size_path);

  if (path[size_path-1] != '\\')
  {
    full_path[size_path] = '\\';
    size_path++;
  }

  memcpy(full_path+size_path, filename, size_filename);
  full_path[size_path+size_filename] = '0';

  return full_path;
}

int utlSaveTextFile(char* file_name, char* text)
{
  FILE* file;

  file = fopen(file_name, "wb");
  if (!file)
    return 0;

  fwrite(text, 1, strlen(text), file);

  if (ferror(file))
  {
    fclose(file);
    return 0;
  }

  fclose(file);
  return 1;
}

char* utlLoadTextFile(char* file_name)
{
  FILE* file;
  char* buffer;
  int size;

  size = utlFileSize(file_name);

  file = fopen(file_name, "rb");
  if (!file)
    return NULL;

  buffer = malloc(size+1);
  fread(buffer, 1, size, file);
  buffer[size] = 0;

  if (ferror(file))
  {
    free(buffer);
    buffer = NULL;
  }

  fclose(file);
  return buffer;
}

char* utlGetFilePath(char *file_name)
{
  char *file_path = utlStrDup(file_name);

  int i = strlen(file_path) - 1;
  while (i != 0)
  {
    if (file_path[i] == '\\' || file_path[i] == '/')
    {
      file_path[i] = 0;
      break;
    }

    i--;
  }

  return file_path;
}

char* utlGetFileTitle(char *file_name)
{
  int i = strlen(file_name) - 1;
  while (i != 0)
  {
    if (file_name[i] == '\\' || file_name[i] == '/')
      break;

    i--;
  }

  if (i == 0 && file_name[0] != '\\' && file_name[0] != '/')
    return utlStrDup(file_name);
  else
    return utlStrDup(file_name + i+1);
}

unsigned long utlFileSize(char* file_name)
{
  FILE* file;
  unsigned long lSize;
  unsigned long lCurrentPosition;

  file = fopen(file_name, "rb");
  if (!file) return 0;

  lCurrentPosition = ftell(file);
  fseek(file, 0L, SEEK_END);
  lSize = ftell(file);
  fseek(file, lCurrentPosition, SEEK_SET);

  fclose(file);

  return lSize;
}

