/*
 * Lua Commander - Load Library Rotines in C
 *
 * Copyright (C) 1999-2000, Antonio E. Scuri, Tecgraf/PUC-Rio.
 * The original file come from Renato Borges LoadLib, 
 * but He also took from someone else in the Internet.
 */

#include <stdlib.h>

/*
  define a macro with your system name (similar to uname:)
  WIN32, HP_UX, Linux, SunOS, IRIX, UNIX_SV, OSF1, SCO_SV, BSD_OS, AIX, Solaris, NeXT
  OR define DL_H or DLFCN_H depending on your system's API to load dynamic libraries
*/

/* systems that support some kind of dynamic linking:
   DLL: WIN32
   SHL: HP-UX
   RLD: NeXT
   DLFCN: Linux, SunOS, IRIX, UNIX_SV, OSF1, SCO_SV, BSD/OS
   DLFCN (simulation): AIX
*/

#if defined(WIN32)

#define DLL

#else
#if defined(HP_UX) || defined(DL_H)

#define SHL

#else
#if defined(Linux)  || defined(SunOS)   || defined(IRIX) || \
    defined(IRIX64) || defined(UNIX_SV) || defined(OSF1) || \
    defined(SCO_SV) || defined(BSD_OS)  || defined(AIX)  || \
    defined(Solaris)|| defined(DLFCN_H)

#define DLFCN

#else
#if defined(NeXT)

#define RLD

#else
#define NODL

#endif
#endif
#endif
#endif

#ifdef DLL
#include <windows.h>
typedef HINSTANCE libtype;
typedef FARPROC   functype;
#define loadlib(path)      LoadLibrary( path )
#define loadfunc(lib,name) GetProcAddress( lib, name )
#define unloadlib(lib)     FreeLibrary( lib )
#endif

#ifdef DLFCN
#include <dlfcn.h>
typedef void* libtype;
typedef void(*functype)(void);
#define loadlib(path)      dlopen( path, RTLD_LAZY | RTLD_GLOBAL )
#define loadfunc(lib,name) (functype)dlsym( lib, name )
#define unloadlib(lib)     dlclose( lib )
#endif

#ifdef SHL
#include <dl.h>
typedef shl_t libtype;
typedef void(*functype)(void);
#define loadlib(path)      shl_load( path, BIND_DEFERRED | BIND_NOSTART, 0L )
#define unloadlib(lib)     shl_unload( lib )

static functype loadfunc( libtype lib, char *name )
{
  functype fn;
  if (shl_findsym( &lib, name, TYPE_PROCEDURE, &fn ) == -1)
    return 0;
  return fn;
}
#endif

#ifdef RLD
#include <rld.h>
typedef long libtype;
typedef void(*functype)(void);
#define loadlib(path)      rldload( 0, 0, path, 0  )
#define unloadlib(lib)     0

static functype loadfunc( libtype lib, char *name )
{
  functype fn;
  char* _name = (char*)malloc((strlen(name)+2)*sizeof(char));
  if (!_name) return 0;
  _name[0] = '_';
  strcpy( _name+1, name );
  if (!rld_lookup( 0, _name, &fn ))
  {
    free(_name);
    return 0;
  }
  free(_name);
  return fn;
}
#endif

#ifdef NODL
typedef void* libtype;
typedef void(*functype)(void);
#define loadlib(path)      (0)
#define loadfunc(lib,name) (0)
#define unloadlib(lib)     ((void)0)
#endif


/*************************************************************************/
/*************************************************************************/

#include <stdio.h>

#include "utl_load_lib.h"


int utlLoadFunc(char *lib_file_name, char *func_name)
{
  libtype lib;
  void (*func)(void);
  char lib_name[10240];                      
#ifdef WIN32
  sprintf(lib_name, "%s.dll", lib_file_name);
#else
  sprintf(lib_name, "lib%s.so", lib_file_name);
#endif

  lib = loadlib(lib_name);
  if (!lib)
    return -1;

  func = (utlLoadedFunc)loadfunc(lib, func_name);
  if (!func)
    return -2;
    
  func();

  return 0;
}

utlLoadedLib* utlLoadLib(char *lib_file_name)
{
  utlLoadedLib* utl_lib;
  libtype lib;

  lib = loadlib(lib_file_name);
  if (!lib)
    return NULL;

  utl_lib = malloc(sizeof(utlLoadedLib));
  utl_lib->lib = lib;

  return utl_lib;
}

utlLoadedFunc utlGetFunc(utlLoadedLib* utl_lib, char *func_name)
{
  return (utlLoadedFunc)loadfunc((libtype)utl_lib->lib, func_name);
}

void utlUnLoadLib(utlLoadedLib* utl_lib)
{
  unloadlib((libtype)utl_lib->lib);
  free(utl_lib);
}

